Unit PathsFiles;

{$MODE OBJFPC}
{$H+}

Interface

Function GetTempDir: String;

Function GetCurrDir: String;

Function FileExists(Filename: String): Boolean;

// Path: If '', use temporary directory; if '.', use current directory
// Prefix: String before random hex filename characters; if '', use 'tmp'
// Extension: File extension for temporary file
// Create: True to create an empty file, or False otherwise
// Returns temporary filename in Filename parameter
// If Create = True: Function returns 0 if it successfully created empty
// file or non-zero if it failed
// If Create = False: Function returns 0
Function GetTempFilenameEx(Path, Prefix, Extension: String; Create: Boolean; Var Filename: String): DWORD;

Implementation

Uses
  Windows;

Const
    INVALID_FILE_ATTRIBUTES = DWORD(-1);

Type
  // This is Win32 API function signature for both GetTempPath and
  // GetCurrentDirectory. Both API functions work identically.
  WIN32GETDIR = Function(nBufferLength: DWORD; lpBuffer: LPTSTR): DWORD; stdcall;

// Call GetTempPath (Which = 0) or GetCurrentDirectory (Which = 1).
Function GetDirectory(Const Which: LongInt): String;
  Var
    GetDirFunc: WIN32GETDIR;
    BufLen, RC: DWORD;
    pDir: LPTSTR;
  Begin
  Result := '';
  Case Which Of
    0: GetDirFunc := @GetTempPath;
  Else
    GetDirFunc := @GetCurrentDirectory;
  End; //Case
  BufLen := GetDirFunc(0, Nil);
  If BufLen <> 0 Then
    Begin
    GetMem(pDir, BufLen);
    RC := GetDirFunc(BufLen, pDir);
    If RC <> 0 Then
      Result := pDir;
    FreeMem(pDir, BufLen);
    End;
  End;

Function GetTempDir: String;
  Begin
  Result := GetDirectory(0);
  End;

Function GetCurrDir: String;
  Begin
  Result := GetDirectory(1);
  End;

Function FileExists(Filename: String): Boolean;
  Begin
  Result := GetFileAttributes(PChar(Filename)) <> INVALID_FILE_ATTRIBUTES;
  End;

Function GetTempFilenameEx(Path, Prefix, Extension: String; Create: Boolean; Var Filename: String): DWORD;
  Var
    TempName: String;
    OK: Boolean;
    F: File;
  Begin
  Result := 0;
  If Path = '' Then
    Path := GetTempDir
  Else If Path = '.' Then
    Path := GetCurrDir;
  If Path[Length(Path)] <> '\' Then
    Path := Path + '\';
  If Prefix = '' Then
    Prefix := 'tmp';
  If Extension <> '' Then
    If Extension[1] <> '.' Then
      Extension := '.' + Extension;
  Randomize;
  Repeat
    TempName := Path + Prefix + HexStr(Random(65535), 4) + Extension;
    OK := Not FileExists(TempName);
    If OK And Create Then
      Begin
      Assign(F, TempName);
      {$I-}
      Rewrite(F);
      Result := IOResult;
      OK := Result = 0;
      If OK Then
        Close(F);
      {$I+}
      End;
  Until OK Or (Result <> 0);
  Filename := TempName;
  End;

Begin
End.
